define( [ '../graph.util', './graph.shape' ], function( util, GraphShape ) {

  function Zoom2DShape() {}

  util.extend( Zoom2DShape.prototype, GraphShape.prototype, {

    createDom: function() {
      this._dom = document.createElementNS( this.graph.ns, 'g' );

      var rect = document.createElementNS( this.graph.ns, 'rect' );
      rect.setAttribute( 'rx', 3 );
      rect.setAttribute( 'ry', 3 );

      rect.setAttribute( 'height', 100 );
      rect.setAttribute( 'width', 6 );
      rect.setAttribute( 'fill', 'rgb(150, 140, 180)' );
      rect.setAttribute( 'stroke', 'rgb( 40, 40, 40 )' );
      rect.setAttribute( 'stroke-width', '1px' );
      rect.setAttribute( 'x', 0 );
      rect.setAttribute( 'y', 0 );

      this.rect = rect;

      this._dom.appendChild( rect );

      var handlePos = document.createElementNS( this.graph.ns, 'rect' );

      handlePos.setAttribute( 'height', 5 );
      handlePos.setAttribute( 'width', 12 );
      handlePos.setAttribute( 'fill', 'rgb(190, 180, 220)' );
      handlePos.setAttribute( 'stroke', 'rgb( 40, 40, 40 )' );
      handlePos.setAttribute( 'stroke-width', '1px' );
      handlePos.setAttribute( 'x', -3 );
      handlePos.setAttribute( 'y', 0 );
      handlePos.setAttribute( 'class', 'positive' );
      handlePos.setAttribute( 'cursor', 'pointer' );

      var handleNeg = document.createElementNS( this.graph.ns, 'rect' );

      handleNeg.setAttribute( 'height', 5 );
      handleNeg.setAttribute( 'width', 12 );
      handleNeg.setAttribute( 'fill', 'rgb(190, 180, 220)' );
      handleNeg.setAttribute( 'stroke', 'rgb( 40, 40, 40 )' );
      handleNeg.setAttribute( 'stroke-width', '1px' );
      handleNeg.setAttribute( 'x', -3 );
      handleNeg.setAttribute( 'y', 0 );
      handleNeg.setAttribute( 'class', 'negative' );
      handleNeg.setAttribute( 'cursor', 'pointer' );

      this._dom.appendChild( handlePos );
      this._dom.appendChild( handleNeg );

      this.handlePos = handlePos;
      this.handleNeg = handleNeg;
    },

    setPosition: function() {
      var position = this._getPosition( this.getFromData( 'pos' ) );

      if ( !position || !position.x || !position.y ) {
        return;
      }

      this.setDom( 'transform', 'translate(' + position.x + ', ' + position.y + ')' );
      return true;
    },

    setHandleNeg: function( value, max ) {

      this.handleNeg.setAttribute( 'y', ( value ) * 45 + 55 );
    },

    setHandlePos: function( value, max ) {

      this.handlePos.setAttribute( 'y', ( 1 - value ) * 45 );
    },

    redrawImpl: function() {
      this.setPosition();
    },

    handleCreateImpl: function() {

      this.resize = true;
      this.handleSelected = 2;

    },

    handleMouseDownImpl: function( e ) {

      this.selected = e.target.getAttribute( 'class' ) == 'positive' ? 'positive' : ( e.target.getAttribute( 'class' ) == 'negative' ? 'negative' : false );
      return true;
    },

    handleMouseUpImpl: function() {

      this.selected = false;
      this.triggerChange();
      return true;
    },

    addSerie: function( serie ) {
      this.series.push( serie );
    },

    handleMouseMoveImpl: function( e ) {

      var o = util.getOffset( this._dom );
      var cY = e.pageY - o.top;
      //console.log( this.selected );

      if ( this.selected == 'negative' ) {

        if ( cY > 100 ) {
          cY = 100;
        } else if ( cY < 55 ) {
          cY = 55;
        }

        //this.handleNeg.setAttribute('y', cY);
        //console.log( cY);
        cY = -( cY - 55 ) / 45;

        this.series.map( function( s ) {
          s.onMouseWheel( false, false, cY, false );
        } );

      }

      if ( this.selected == 'positive' ) {

        if ( cY < 0 ) {
          cY = 0;
        } else if ( cY > 45 ) {
          cY = 45;
        }

        // this.handlePos.setAttribute('y', cY);
        cY = ( 45 - cY ) / 45;

        this.series.map( function( s ) {
          s.onMouseWheel( false, false, cY, true );
        } );

      }

    },

    selectStyle: function() {
      this.setDom( 'stroke', 'red' );
      this.setDom( 'stroke-width', '2' );
    },

    hideHandleNeg: function() {
      this.handleNeg.setAttribute( 'display', 'none' );
      this.rect.setAttribute( 'height', 45 );
    },

    showHandleNeg: function() {
      this.handleNeg.setAttribute( 'display', 'block' );
      this.rect.setAttribute( 'height', 100 );
    },

    setHandles: function() {}

  } );

  return Zoom2DShape;

} );
